/** Platform system **/

/** The platform movement system. It works inside an UpdateScript. */
function PlatformSystem(person)
{
  this.person = person;
  var p = this.person;
  this.active = true; //For pausing/unpausing the platform system.
  this.control = true; //Does the player have control? (useful for events)
  
  this.enterx = 0;    //X and Y position you entered the map from.
  this.entery = 0;
  
  //Player's stats
  this.jumpstrength = 2;
  this.strength = 2;  //Meaning: can move 2 light boxes or 1 normal box.
  
  var jumping = false; //are we jumping?
  var yspeed = 0; //speed of falling.
  var gravity = 0.2; //yspeed modifier
  
  var jumpanim = false;
  
  /** Initialize stuff. */
  this.init = function()
  {
    SetPersonFrameRevert(p, 1);
    this.reset();
  }
  
  /** Reset platform system. Does NOT reset powerups. */
  this.reset = function()
  {
    jumping = false; //are we jumping?
    yspeed = 0; //speed of falling.
    
    jumpanim = false;
    UpdateMapEngine();
  }
  
  /** This is what you execute in an UpdateScript. */
  this.run = function()
  {
    if (!this.active) return;
    var x = GetPersonX(p);
    var y = GetPersonY(p);
    var dir = GetPersonDirection(p).slice(0, 4);
    var xspeed = (dir == "east") ? 2 : -2;
    
    //*********************
    //HORIZONTAL MOVEMENT
    if (IsKeyPressed(KEY_LEFT) && this.control)
    {
      QueuePersonCommand(p, COMMAND_MOVE_WEST, true);
      QueuePersonCommand(p, COMMAND_MOVE_WEST, true);
      QueuePersonCommand(p, COMMAND_ANIMATE, true);
      dir = "west";
    }
    
    if (IsKeyPressed(KEY_RIGHT) && this.control)
    {
      QueuePersonCommand(p, COMMAND_MOVE_EAST, true);
      QueuePersonCommand(p, COMMAND_MOVE_EAST, true);
      QueuePersonCommand(p, COMMAND_ANIMATE, true);
      dir = "east";
    }
    
    //Slope
    if (IsPersonObstructed(p, x+xspeed, y) && !IsPersonObstructed(p, x+xspeed, y-2)
        && (IsKeyPressed(KEY_LEFT) || IsKeyPressed(KEY_RIGHT)) && this.control)
    {
      y-=3;
    }
    
    //*********************
    //TOUCHING STUFF
    var op = GetObstructingPerson(p, x+xspeed, y+yspeed);
    
    /*if (op != "") AfterRender = function() { FONT.drawText(100, 5, "Touching " + a); }
    else AfterRender = function() { FONT.drawText(100, 5, "Touching nothing."); }*/
    
    if (op != "")
    {
      if (op.slice(4, 10) == "deadly")
      {
        this.die();
        return;
      }
      
      //Touch powerup
      else if (op.slice(0, 16) == "powerup-strength")
        GetStrengthPowerup(op.slice(17, op.length));
      else if (op.slice(0, 12) == "powerup-jump")
        GetJumpPowerup(op.slice(13, op.length));
      
      //Touch button = spring
      else if (op.slice(0, 6) == "button")
        yspeed = this.jumpstrength * -4,
        jumpanim = true;
      
      var hop = GetObstructingPerson(p, x+xspeed, y);  //Horiz only, for boxes.
      if (hop.slice(0, 3) == "box")
      {
        var nextbox = hop;
        //Loop: max strength+1 lightboxes can be pushed when next to eachother.
        for (var i = 0; i < this.strength+1;)
        {
          //i works as a "weight counter": add weight until the max is reached.
          //If the max is reached (i>this.strength+1) then it's impossible to
          //move more horizontally stacked blocks.
          if (nextbox.slice(4, 9) == "light") i++;
          else if (nextbox.slice(4, 10) == "normal") i+=2;
          else if (nextbox.slice(4, 9) == "heavy") i+=3;
          else break;  //Unmovable box
          if (i >= this.strength+1) break;
          
          //Yes, move it twice.
          for (var b = 0; b < 1; b++)
          {
            if (xspeed > 0) QueuePersonCommand(nextbox, COMMAND_MOVE_EAST, true);
            else QueuePersonCommand(nextbox, COMMAND_MOVE_WEST, true);
          }
          var nextx = GetPersonX(nextbox);
          var nexty = GetPersonY(nextbox);
          nextbox = GetObstructingPerson(nextbox, nextx+xspeed, nexty);
          
          if (nextbox == "") break;
        }
      }
    }
    
    //*********************
    //VERTICAL MOVEMENT
    //FALLING AND BOTTOM OBSTRUCTIONS:
    if (!IsPersonObstructed(p, x, y+yspeed))
    {
      yspeed += gravity;
      y += yspeed;
      if (yspeed > 0) jumping = true;
      else jumping = false;
    }
    
    //Falling (and hopefully jumping): obstructions.
    if (IsPersonObstructed(p, x, y+yspeed))
    {
      var temp = 0;
      for (i = 0; i < y+yspeed; i++)
      {
        if (IsPersonObstructed(p, x, y+i))
        {
          temp++;
          break;
        }
      }
      temp--;
      if (jumping) temp *= -1;
      y += temp;
      yspeed = 0;
      if (jumping) yspeed = 1;  //So we fall down a bit faster when we hit a ceiling.
      
      if (jumping) jumpanim = false;
    }
    
    //JUMPING:
    if (IsKeyPressed(KEY_JUMP) && IsPersonObstructed(p, x, y+yspeed) && this.control)
    {
      yspeed = this.jumpstrength * -2;
      jumpanim = true;
    }
    
    SetPersonY(person, y);
    jump = (jumpanim)?"-jump":"";
    SetPersonDirection(p, dir+jump);
  }
  
  this.jump = function(strength)
  {
    yspeed = this.jumpstrength * (strength*-1);
    jumpanim = true;
  }
  
  this.die = function()
  {
    this.active = false;  //Turn off the platform engine.
    var dyspeed = -15;
    DetachCamera();
    var textwidth = DEATHFONT.getStringWidth("You Die.");
    var textx = (GetScreenWidth() - textwidth) / 2;
    
    for (var t = 0; t < 120; t++)
    {
      dyspeed++;
      SetPersonY(p, GetPersonY(p)+dyspeed);
      UpdateMapEngine(), RenderMap();
      DEATHFONT.setColorMask(black);
      DEATHFONT.drawText(textx-1, 101, "You Die.");
      DEATHFONT.setColorMask(white);
      DEATHFONT.drawText(textx, 100, "You Die.");
      FlipScreen();
    }
    LIVES--;
    
    if (LIVES < 0)
    {
      while (!IsKeyPressed(KEY_ESCAPE))
      {
        RenderMap();
        Rectangle(0, 0, GetScreenWidth(), GetScreenHeight(), trans);
        DEATHFONT.setColorMask(black);
        DEATHFONT.drawText(textx-21, 101, "Game Over");
        DEATHFONT.setColorMask(red);
        DEATHFONT.drawText(textx-20, 100, "Game Over");
        FlipScreen();
      }
      RestartGame();
    }
    
    this.active = true;
    ChangeMap(GetCurrentMap());  //Just reload the map.
    SetPersonX(p, this.enterx);
    SetPersonY(p, this.entery);
    AttachCamera(p);
  }
}


/** Check the gravity of all persons on the map and let them fall down if necessary. */
function CheckGravity()
{
  var list = GetPersonList();
  for (p in list)
  {
    if (list[p].slice(0, 3) == "box")
    {
      if (!IsPersonObstructed(list[p], GetPersonX(list[p]), GetPersonY(list[p])+2))
        QueuePersonCommand(list[p], COMMAND_MOVE_SOUTH, true),
        QueuePersonCommand(list[p], COMMAND_MOVE_SOUTH, true);
    }
  }
}
